#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "tr.h"
#include "msd.h"
#include "db.h"
#include "stock.h"

typedef struct Flags { 
    unsigned int verbose : 4 ; 
    unsigned int dump : 4 ; 
    unsigned int hex : 4 ; 
    unsigned int opaque : 1 ; 
} Flags ;

static void
usage ()
{
    cbanner ( 0,
	     "[-dvx] [miniseed ..]", 
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}
    
void
show_opaque ( Tbl *opaque, Flags flags ) 
{
    Msd_opaque *b ; 
    int i, j ; 
    for (i=0 ; i<maxtbl(opaque) ; i++ ) {
	b = (Msd_opaque *) gettbl(opaque, i) ; 
	printf ( "Opaque #%d  record #%d  %d bytes %s  flags=%x\n", 
		i, 
		b->record,
		b->nbytes,
		b->sparc_order ? "sparc" : "intel",
		b->flags ) ; 
	for (j=0 ; j<maxtbl(b->fields) ; j++ ) {
	    printf ( "    %2d %s\n", j, (char *) gettbl(b->fields, j) ) ; 
	}
	if (flags.verbose) { 
	    hexdump(stdout, b->data, b->nbytes) ; 
	}
    }
}

void
show_b1001(Msd *msd) 
{
    if ( msd->flags.has_b1001) { 
	printf ( "  B1001: timing_quality=%d  usec=%d  frame_count=%d\n", 
	    msd->b1001.timing_quality, 
	    msd->b1001.usec, 
	    msd->b1001.frame_count ) ;  
    }
}

void
show_data ( Msd *msd ) 
{
    long i ; 
    static int *data ; 
    float *dp ; 
    long npts = 0 ; 
    int umsd_result ;
    umsd_result = umsd ( msd, &data, &npts ) ;
    switch ( umsd_result ) { 
	case 0:
	    printf ( "    " ) ; 
	    for ( i=0 ; i<npts ; i++ ) { 
		printf ( " %8d", data[i] ) ; 
		if ( ((i+1) % 5) == 0 ) { 
		    printf ( "\n    " ) ; 
		}
	    }
	    if ( ((i) % 5) != 0 ) {
		printf ( "\n" ) ; 
	    }
	    break; 

	case MSD_FLOAT_DATA:
	    printf ( "    " ) ; 
	    dp = (float *) data ;
	    for ( i=0 ; i<npts ; i++ ) { 
		printf ( " %.8g", dp[i] ) ; 
		if ( ((i+1) % 5) == 0 ) { 
		    printf ( "\n    " ) ; 
		}
	    }
	    if ( ((i) % 5) != 0 ) {
		printf ( "\n" ) ; 
	    }
	    break; 

	 default:
	    complain ( 0, "umsd failed" ) ; 
	    break; 
    }
}

void
show_msd ( Msd *msd, Flags flags) 
{
    char net[16], sta[16], chan[16], loc[16] ; 
    char filename[FILENAME_MAX] ; 
    double samprate, nominal, time ; 
    int nsamp, has_sdh, has_b100, has_b1000, has_b1001 ;
    int dataformat, fpr, fdf, sparc_order, activity, io, 
    	level, quality, samprate_flags, record_size, correction ; 
    int has_opaque ; 
    char nopaque[32] ;
    Tbl *opaque ;
    char *s ;

    msdget ( msd, 
	MSD_NET, net, 
    	MSD_STA, sta, 
	MSD_CHAN, chan, 
	MSD_LOC, loc, 
	MSD_SAMPRATE, &samprate, 
	MSD_NOMINAL_SAMPRATE, &nominal,
	MSD_TIME, &time, 
	MSD_NSAMP, &nsamp, 
	MSD_HAS_SEED_HEADER, &has_sdh, 
	MSD_HAS_B100, &has_b100, 
	MSD_HAS_B1000, &has_b1000, 
	MSD_HAS_B1001, &has_b1001, 
	MSD_LEVEL, &level,
	MSD_DATAFORMAT, &dataformat, 
	MSD_FRAMES_PER_RECORD, &fpr, 
	MSD_FIRST_DATA_FRAME, &fdf, 
	MSD_SPARC_ORDER, &sparc_order, 
	MSD_FILENAME, filename, 
	MSD_ACTIVITY_FLAGS, &activity, 
	MSD_IO_FLAGS, &io, 
	MSD_QUALITY_FLAGS, &quality, 
	MSD_SAMPRATE_FLAGS, &samprate_flags, 
	MSD_RECORD_SIZE, &record_size, 
	MSD_TIME_CORRECTION, &correction, 
	MSD_HAS_OPAQUE, &has_opaque,
	MSD_OPAQUE, &opaque,
         0 ) ; 

    sprintf(nopaque, " opaque(%ld)", maxtbl(opaque) ) ; 
    printf ( "\n%2s %5s %3s %2s  %s  %5d %10.4f", 
    		net, sta, chan, loc, s=strydtime(time), nsamp, samprate ) ; 
    free(s) ; 
    if ( nominal != samprate ) { 
	printf ( " (%8.4f nominal)", nominal ) ; 
    }
    printf ( "\n" ) ; 
    printf ( "    %5d b/rcd %2d fpr %2d fdf (%s) %s%s%s%s\n", 
		record_size, fpr, fdf,
		sparc_order ? "sparc" : "intel", 
    		has_b1000 ? " B1000" : "",
    		has_b1001 ? " B1001" : "",
    		has_b100 ? " B100" : "", 
		has_opaque ? nopaque : "") ; 
    printf ( "    dataformat %2d Steim level %d  \n", 
    	dataformat, level ) ; 
    printf ( "           io   activity quality samprate time-correction\n" ) ; 
    printf ( "  flags: %4x    %4x     %4x    %4x      %10d\n", 
    	io, activity, quality, samprate_flags, correction ) ;
    	
    if ( flags.verbose ) { 
	show_b1001(msd) ;
	if ( flags.verbose > 1) { 
	    show_data ( msd ) ; 
	}
    }
    if ( flags.opaque && has_opaque ) { 
	show_opaque ( opaque, flags ) ; 
    }
}


int
main (int argc, char **argv)
{
    int             c,
		    errflg = 0;
    Msd		   *msd ;
    Flags	    flags ;

    memset ( &flags, 0, sizeof(flags)) ;
    elog_init ( argc, argv ) ; 

    while ((c = getopt (argc, argv, "dovVx")) != -1) {
	switch (c) {

	case 'd':
	    flags.dump++ ; 
	    break ;

	case 'o':
	    flags.opaque = 1 ; 
	    break; 

	case 'x':
	    flags.hex++ ; 
	    break ;

	case 'v':
	    flags.verbose++ ;
	    break;

	case 'V':
	    usage ();
	    break;

	case '?':
	    errflg++;
	    break ;
	}
    }

    if (errflg)
	usage ();

    msd = msdnew() ;
    if ( argc - optind > 0 ) { 
	for (; optind < argc; optind++) {
	    char *path, *data ; 
	    long size ;
	    path = argv[optind] ;
	    printf ( "\n%s\n", path ) ; 
	    size = mapfile ( path, 0,  (char **) &data ) ; 
	    msdput ( msd, MSD_FILENAME, path, 
			MSD_MSEED, data, 
			MSD_MSEED_NBYTES, size, 
			0 ) ;
	    while ( msdnxt(msd) ) { 
		if (flags.dump) { 
		    dmsd(stdout, msd, flags.verbose ) ; 
		} else { 
		    show_msd (msd, flags) ; 
		}
		if (flags.hex) { 
		    char *mseed ; 
		    int record_size ;
		    msdget(msd, 
			MSD_RECORD, &mseed, 
			MSD_RECORD_SIZE, &record_size, 
			0 ) ;
		    hexdump(stdout, mseed, record_size) ;
		}
	    }
	}
    } else { 
	msdput ( msd, MSD_FD, 0, 0 ) ; 
	while ( msdnxt(msd) ) {
	    show_msd (msd, flags) ; 
	}
    }

    return 0;
}
